from flask import Flask, render_template, request, jsonify
from flask_ngrok import run_with_ngrok 
import cv2
import dlib
import numpy as np
import pickle
import base64

app = Flask(__name__)

# Load model deteksi wajah dlib
detector = dlib.get_frontal_face_detector()
sp = dlib.shape_predictor("shape_predictor_68_face_landmarks.dat")
facerec = dlib.face_recognition_model_v1("dlib_face_recognition_resnet_model_v1.dat")

# Load model SVM dan Label Encoder
with open("svm_model.pkl", "rb") as f:
    svm = pickle.load(f)

with open("label_encoder.pkl", "rb") as f:
    label_encoder = pickle.load(f)

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/detect_faces', methods=['POST'])
def detect_faces():
    try:
        data = request.json
        image_data = data.get("image")

        if not image_data:
            return jsonify({"error": "No image data received"}), 400

        # Decode base64 ke image
        image_bytes = base64.b64decode(image_data.split(",")[1])
        np_arr = np.frombuffer(image_bytes, np.uint8)
        frame = cv2.imdecode(np_arr, cv2.IMREAD_COLOR)

        rgb_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
        faces = detector(rgb_frame)

        detected_names = []
        for face in faces:
            shape = sp(rgb_frame, face)
            face_descriptor = facerec.compute_face_descriptor(rgb_frame, shape)
            face_descriptor = np.array(face_descriptor).reshape(1, -1)

            pred_label = svm.predict(face_descriptor)[0]
            pred_name = label_encoder.inverse_transform([pred_label])[0]

            detected_names.append(pred_name)

        return jsonify({"detected_faces": detected_names})

    except Exception as e:
        return jsonify({"error": str(e)}), 500

if __name__ == "__main__":
    app.run()
